-----------------------------------------------------------------------------------------
-- Start Activity
-----------------------------------------------------------------------------------------
-- Script is the main part of the mission. You can't generally create a mission without some Lua
-- so get ready to some light to medium programming and add http://lua.org , Cortex Command Wiki
-- and forum search to the favourites in your browser.
--
-- This method\function is the place where the mission starts. Here you define and initialize variables, 
-- create scripted objects (brains, actors etc) and so on. If something goes wrong in this method, then
-- you kicked out to the campaign screen with error message in the console. Some times you need to restart
-- your mission twice, because CC will keep kicking you out complaining about already fixed errors.
-- Notice the name of the function, SweetDreams part is that Lua class name that you used in mission activities.ini
function SweetDreams:StartActivity()
	-- Always print debugging information to the console but keep in mind that heavy prinintg
	-- causes heavy lag. Comment all the unnecessary prints when you release your mission to the public.
	-- If you notice some heavy unexpected lag, check the console. CC is probably flooding it with error messages.
	print("START! -- SweetDreams:StartActivity()!");

	-- AWESOMENESS THAT'S FOR YOU!!! :)
	-- To prevent CC from offering all available scenes to play we need to define our very special 
	-- area in the scene for our mission and then request that area in the script. CC compares
	-- area requests in scripts and actual areas in scenes and select playable scenes for activity.
	self.Zone = SceneMan.Scene:GetArea("SweetDreams1Unique");
	
	-- Array which stores flags for players that lost their brain.
	self.braindead = {};
	-- Array which stores brains assigned to players. It is used for quick enumeration of 
	-- playable brains to trigger something or show mission marks.
	self.AssignedBrains = {};
	-- This is used to limit the amount of ingame movable objects (MO's). Currently CC goes mad if this number exceeds
	-- 255. To avoid unexpected behaviour you MUST always take care of the number of the movable ID's. We'll talk more
	-- about movable objects and limits later when we'll discuss spawning. Further reading - http://www.datarealms.com/wiki/index.php/Scenes
	self.MOIDLimit = 200;
	
	-- Here we store the amount of gold that player will recieve at start. Note that we do not actually give
	-- the player real gold here.
	self.PlayerGold = 300; -- Player starting credits, in this case lives.

   	-- Team variables are used just for convenience.
   	-- Team 2 is always the CPU.
	self.PlayerTeam = Activity.TEAM_1;
   	self.CPUTeam = Activity.TEAM_2;
	self.RogueTeam = -1;
	
	-- Define enumeration with phases of mission. These are just friendly names for numbers 0,1,2,3 used to designate
	-- current mission phase. Nobody can stop you from using actual numbers (known as "magic numbers", because you can't
	-- determine what and how this number affects), but it is considered bad taste. Try using named variables and enumerations
	-- everywhere possible.
	self.FightPhase = { NOFIGHT = 0 , PHASEONE = 1 , PHASETWO = 2 , PHASETHREE = 3 };
	-- Set initial phase of mission.
	self.CurrentFightPhase = self.FightPhase.NOFIGHT;
	
	-- The last position of Dimitri.
	self.lastpos = nil;
	-- The stage of the intro.  It's negative because I needed to add stuff to the beginning, (skip text) and
	-- I didn't want to have to rewrite all my stages.
	self.IntroStage = -2;
	-- The timer that times the stages of the intro.
	self.IntroTimer = Timer();
	-- Times the animation of Dimitri sleeping.
	self.SleepTimer = Timer();
	-- Tells us if we've picked up the note yet.
	self.TakenNote = false;
	
	-- Likewise for the outro.
	self.OutroStage = -2;
	self.OutroTimer = Timer();
	self.DoorTimer = Timer();
	-- For cutscenes that don't start at the very beginning of the mission,
	-- actors who have been controlled (I think that's the cause) always behave
	-- on sentry.  So I have to set their aim angle every frame in the outro
	-- so he doesn't look up and down.  So if I tell an actor to look up, but
	-- not every frame, he'll look up for less than a second, until the actor is
	-- told by the sentry AI to look up or down again.  Weird.
	self.ViewAngle = 0;
	
	-- Turns to true when you've killed all the coalition
	self.KilledSoldiers = false;
	
	self.Door = CreateMOSRotating("SweetDreams.rte/Castle Door");
	self.Door.Pos = Vector(3700, 590);
	MovableMan:AddParticle(self.Door);
	self.DoorBox = Box(3700-(128/2), 590-(160/2), 3700+(128/2),590+(160/2));
	
	-- Timer for flashing animation.
	self.FlashTimer = Timer();
	self.Flashing = false;
	
	-- If Dimitri needs to switch weapons next frame.
	self.WeaponSwitch = false;
	
	-- Whether the outro has been skipped.
	self.SkippedOutroYet = false;
		
	-- (Below is a comment from the tutorial.)
   	-- Set up players
	-- Here we create brains for players. Although current CC doesn't have predefined activities with multiplayer missions,
	-- keep in mind that this behaviour may change, so you should treat gameplay as it was a multiplayer game so you'll have
	-- to fix less when CC will support standard multiplayer activities for missions.
	for player = 0, self.PlayerCount - 1 do
		
		--Made a copy of Dimitri because Ronin will be taken out eventually- and I can change stuff now too
		self.PlayerActor = CreateAHuman("Dimitri Soldier","SweetDreams.rte");
		self.PlayerActor.AIMode = Actor.AIMODE_NONE;
		self.PlayerActor:SetControllerMode(Controller.CIM_NETWORK,-1);
		self.PlayerActor.Team = Activity.TEAM_1;
		self.Pies = CreateHDFirearm("Pies","SweetDreams.rte");
		self.PlayerActor:AddInventoryItem(self.Pies);
		self.PlayerActor.Pos = Vector(45,960);
		controller = self.PlayerActor:GetController();
		
		MovableMan:AddActor(self.PlayerActor);
		
		-- Create an invisible brain.  I'm doing this because the game crashes without a brain, and because I need
		-- for it to look at the sleeping bag before I move Dimitri there.
		self.brain = CreateActor("SweetDreams.rte/Invisible Brain Case");
		self.brain.Team = Activity.TEAM_1;
		self.brain.Pos = Vector(110,560);
		self.brain:SetAimAngle(math.pi / -5)
		self.brain.HFlipped = true;
		self.brain.Scale = 0;
		--self.brain.GetsHitByMOs = false;
		self.brain.HitsMOs = false;
		MovableMan:AddActor(self.brain);
		self:SetPlayerBrain(self.brain,Activity.PLAYER_1);
		self:SwitchToActor(self.brain,Activity.PLAYER_1,-1);
		
		-- Force the player to control the invisible brain (at first)
		self.ForcedControlledActor = self.brain;
		
		-- Used to loop through all the checkpoints
		self.Checkpoints = {};
		-- I can't find support for getting the length of a table, so...
		self.CheckpointAmount = 0;
		
		self.CP1 = CreateMOSRotating("SweetDreams.rte/Checkpoint");
		self.CP1.Pos = Vector(1000,601);
		self.CP1.Team = Activity.TEAM_1;
		MovableMan:AddParticle(self.CP1);
		table.insert(self.Checkpoints, 0, self.CP1);
		self.CheckpointAmount = self.CheckpointAmount + 1;
		
		self.CP2 = CreateMOSRotating("SweetDreams.rte/Checkpoint");
		self.CP2.Pos = Vector(2382,617);
		self.CP2.Team = Activity.TEAM_1;
		MovableMan:AddParticle(self.CP2);
		table.insert(self.Checkpoints, 0, self.CP2);
		self.CheckpointAmount = self.CheckpointAmount + 1;
		
		-- Sleeping bag
		self.DimitriBag = CreateMOSRotating("SweetDreams.rte/Sleeping Bag");
		self.DimitriBag.Pos = Vector(110,559);
		self.DimitriBag.Team = Activity.TEAM_1;
		MovableMan:AddParticle(self.DimitriBag);
		
		-- 1st tutorial screen
		self.Screen = CreateMOSRotating("SweetDreams.rte/Screen");
		self.Screen.Pos = Vector(540,627);
		self.Screen.Team = Activity.TEAM_1;
		MovableMan:AddParticle(self.Screen);
		
		self.Note = CreateHeldDevice("Note","SweetDreams.rte");
		self.Note.Pos = Vector(175 , 570)
		MovableMan:AddItem(self.Note);
		
		self:AddCoalition(1260, 613, true, false, "Pistol");
		self:AddCoalition(1741, 646, true, false, "Shotgun");
		self:AddCoalition(1988, 545, true, false, "Sniper Rifle");
		self:AddCoalition(2282, 533, false, true, "Heavy Sniper Rifle");
		self:AddCoalition(2114, 650, true, true, "Mauler Shotgun");
		self:AddCoalition(1964, 475, true, true, "Auto Shotgun");
		--self:AddCoalition(3200, 635, true, true, "Heavy Sniper Rifle"); --Taken out, didn't like it
	end
	
	
	-- Set the team's funds. Here we actually give a player some gold.
	self:SetTeamFunds(self.PlayerGold , self.PlayerTeam);
	
	-- While developing, I'll skip the intro (comment this to keep the intro)
	-- NOT ANYMORE!  I made a skip button!
	--self:AdvanceToOne();
	--self.PlayerActor.Pos = Vector(110,550); -- Bring Dimitri back from the LOL cave
	
	-- While developing, I'll skip having to kill all the soldier just to see the outro
	--self:AdvanceToTwo();
end
-----------------------------------------------------------------------------------------
-- Pause Activity
-----------------------------------------------------------------------------------------
-- This function is called when mission is paused
function SweetDreams:PauseActivity(pause)
	print("PAUSE! -- SweetDreams:PauseActivity()!");
end
-----------------------------------------------------------------------------------------
-- End Activity
-----------------------------------------------------------------------------------------
-- This function is called after mission has ended
function SweetDreams:EndActivity()
	print("END! -- SweetDreams:EndActivity()!");
end
-----------------------------------------------------------------------------------------
-- Update Activity
-----------------------------------------------------------------------------------------
function SweetDreams:UpdateActivity()

   	-- Clear all objective markers, they get re-added each frame
   	self:ClearObjectivePoints();
   	
   	if self.WeaponSwitch == true then
   		self.PlayerActor:GetController():SetState(Controller.WEAPON_CHANGE_PREV,true);
   		self.WeaponSwitch = false;
   	end
   	
   	-- Keep the brain's health invisible.
	self.brain.Team = self.RogueTeam;


	if self.CurrentFightPhase == self.FightPhase.NOFIGHT then
		self:PhaseNoFight();
	elseif self.CurrentFightPhase == self.FightPhase.PHASEONE then
		self:PhaseOne();
	elseif self.CurrentFightPhase == self.FightPhase.PHASETWO then
		self:PhaseTwo();
	elseif self.CurrentFightPhase == self.FightPhase.PHASETHREE then
		self:PhaseThree();
	end
	
	-- Make sure the player doesn't switch to anyone else; we don't want them to see Dimitri underground or the invisible brains when they shouldn't XD
	if self.ActivityState ~= Activity.OVER then
		local Activity = ActivityMan:GetActivity();
		local CurrentActor = Activity:GetControlledActor(0);
		if CurrentActor == nil then
			CurrentActor = self.brain;
		end
		if CurrentActor.ID ~= self.ForcedControlledActor.ID then
			Activity:SwitchToActor(self.ForcedControlledActor, 0, 0);
			if self.CurrentFightPhase == self.FightPhase.NOFIGHT then
				-- Fixes the intro, if you try to switch, this gets reset because it gets turned off when you switch to control the actor.
				self.PlayerActor:SetControllerMode(Controller.CIM_NETWORK,-1);
			end
		end
	end
	
	-- Really bored.  This snippet adds falling bodies for added challenge XD
	-- It uses the scene width in its code to make sure no matter what the
	-- width of the map is, the bodies rain at the same density.
	
	--if self.BodyTimer == nil then
	--	self.BodyTimer = Timer();
	--end	
	--if self.BodyTimer:IsPastSimMS(1000000/SceneMan.SceneWidth) then
	--	self.BodyTimer:Reset();
	--	self.Body = CreateAHuman("Dimitri","Ronin.rte");
	--	self.Body.AIMode = Actor.AIMODE_SENTRY;
	--	self.Body:SetControllerMode(Controller.CIM_NETWORK,-1);
	--	self.Body.Team = Activity.TEAM_2;
	--	self.Body.Health = 0
	--	self.Body.Pos = Vector(math.random(SceneMan.SceneWidth),1);
	--	self.Body.Vel = Vector(math.random(-6,6), math.random(50))
	--	MovableMan:AddActor(self.Body);
	--end
	
	
	-- Check if we lost or won
	self:Check();
	
	self:YSortObjectivePoints();
	
	
end
-----------------------------------------------------------------------------------------
-- No Fight (Intro)
-----------------------------------------------------------------------------------------
-- This phase is where I put the intro.
function SweetDreams:PhaseNoFight()
	
	if UInputMan:KeyPressed(26) then
		self:SkipIntro();
	end
	if self.IntroStage == -2 then -- (I didn't want to have to rearrange the numbers)
		self:ResetMessageTimer(0);
		FrameMan:SetScreenText("(Press 'z' to skip)", 0, 0, 1000, false);
		self.IntroStage = -1;
	end
	if self.IntroTimer:IsPastSimMS(2000) and self.IntroStage == -1 then
		FrameMan:ClearScreenText(0);
		self.IntroStage = 0;
	end
	-- Snore snore snore
	if self.SleepTimer:IsPastSimMS(900) and self.IntroStage <= 0 then
		if self.DimitriBag.Frame == 0 then
			self.DimitriBag.Frame = 1;
		else
			self.DimitriBag.Frame = 0;
		end
		self.SleepTimer:Reset();
	end
	
	-- Talk talk talk
	if self.IntroTimer:IsPastSimMS(5000) and self.IntroStage == 0 then
		self.IntroStage = 1;
		self.DimitriBag.Frame = 3;
		self.PlayerActor.Pos = Vector(110,550); -- Bring Dimitri back from the LOL cave 
		self:ResetMessageTimer(0);
		FrameMan:SetScreenText("*Yawn*", 0, 0, 3000, false);
		--self:SwitchToActor(self.PlayerActor,Activity.PLAYER_1,-1);
	end
	--Look at the note slightly before cursing, so it looks natural
	if self.IntroTimer:IsPastSimMS(9000) and self.IntroStage == 1 then
		self.PlayerActor:SetAimAngle(-0.35);
	end
	if self.IntroTimer:IsPastSimMS(10000) and self.IntroStage == 1 then
		self.IntroStage = 2;
		self:ResetMessageTimer(0);
		FrameMan:SetScreenText("Damnit.", 0, 0, 3000, false);
	end
	if self.IntroTimer:IsPastSimMS(15000) and self.IntroStage == 2 then
		self.IntroStage = 3;
		self:ResetMessageTimer(0);
		FrameMan:SetScreenText("My cake is gone... Replaced with a note.", 0, 0, 1500, false);
	end
	
	-- Walk walk wa- No, this isn't very self explanatory.  He walks until he is on top of the note
	-- and then picks it up.  The intro stage is changed and the timer is reset.  I reset it
	-- because depending on how people have their delta set (their framerate) their physics could vary,
	-- changing the time it takes to get there.
	if self.IntroStage == 3 then
		if self.PlayerActor.Pos.X <= 170 then
			self.PlayerActor:GetController():SetState(Controller.MOVE_RIGHT, true);
		elseif self.TakenNote == false then
			controller:SetState(Controller.WEAPON_PICKUP,true);
			self.TakenNote = true; --Make sure he doesn't try to pick it up again
			self.IntroStage = 4;
			self.IntroTimer:Reset(); --Reset the timer; after all, the time by now could vary
			self.PlayerActor:SetAimAngle(0) -- Look forward as if to read the paper
		end
	end
	
	-- Read read read, but wait a second after he picks it up so it looks natural
	if self.IntroTimer:IsPastSimMS(1000) and self.IntroStage == 4 then
		self.IntroStage = 5;
		self:ResetMessageTimer(0);
		FrameMan:SetScreenText("\"deer dimytree lol we steeled ur caik wile u waz sleepin.", 0, 0, 3000, false);
	end
	if self.IntroTimer:IsPastSimMS(6000) and self.IntroStage == 5 then
		self.IntroStage = 6;
		self:ResetMessageTimer(0);
		FrameMan:SetScreenText("iz gona be so taystee haha zomgwtfbbq. luvz cowalishun\"", 0, 0, 3000, false);
	end
	if self.IntroTimer:IsPastSimMS(13000) and self.IntroStage == 6 then
		self.IntroStage = 7;
		self:ResetMessageTimer(0);
		FrameMan:SetScreenText("Those Coalition bastards are trying to ruin my birthday!", 0, 0, 3000, false);
		-- Rip the note
		self.RipSound = CreateAEmitter("SweetDreams.rte/Rip Sound");
		self.RipSound.Pos = self.Note.Pos;
		MovableMan:AddParticle(self.RipSound);
		self.ScrapRight = CreateMOSRotating("Paper Right");
		self.ScrapRight.Pos = self.PlayerActor.Pos + Vector(4,0);
		self.ScrapRight.Vel = Vector(2,0);
		MovableMan:AddMO(self.ScrapRight);
		self.ScrapLeft = CreateMOSRotating("Paper Left");
		self.ScrapLeft.Pos = self.PlayerActor.Pos + Vector(2,0);
		self.ScrapLeft.Vel = Vector(5,0);
		MovableMan:AddMO(self.ScrapLeft);
		self.Note.ToDelete = true;
	end
	
	-- Wait a frame becase you can't do ToDelete and weapon change on the same frame- a glitch will allow
	-- the player to switch to nothing.
	if self.IntroTimer:IsPastSimMS(13001) and self.IntroStage == 7 then
		self.PlayerActor:GetController():SetState(Controller.WEAPON_CHANGE_PREV,true);
	end
	if self.IntroTimer:IsPastSimMS(19000) and self.IntroStage == 7 then
		self.IntroStage = 8;
		self:ResetMessageTimer(0);
		FrameMan:SetScreenText("I shall reclaim my cake and feast on it at the party tonight!", 0, 0, 3000, false);
	end
	
	if self.IntroTimer:IsPastSimMS(24000) and self.IntroStage == 8 then
		self:AdvanceToOne();
	end
end
-----------------------------------------------------------------------------------------
-- Advance To One
-----------------------------------------------------------------------------------------
function SweetDreams:AdvanceToOne()
	-- Switch to next phase
	self.CurrentFightPhase = self.FightPhase.PHASEONE;
	
	-- The intro is over, control Dimitri
	self.ForcedControlledActor = self.PlayerActor;
end

-----------------------------------------------------------------------------------------
-- Phase One
-----------------------------------------------------------------------------------------
--This is where most of the gameplay takes place.
function SweetDreams:PhaseOne()
	self:ObjectiveSweep();
	if controller:IsState(Controller.WEAPON_PICKUP) then
		controller:SetState(Controller.WEAPON_PICKUP, false);
		FrameMan:ClearScreenText(0);
		self:ResetMessageTimer(0);
		FrameMan:SetScreenText("I don't know how to use a gun...", 0, 0, 3000, false);
	end
	if controller:IsState(Controller.WEAPON_DROP) then
		controller:SetState(Controller.WEAPON_DROP, false);
		FrameMan:ClearScreenText(0);
		self:ResetMessageTimer(0);
		FrameMan:SetScreenText("Why would I ever drop my only weapon?", 0, 0, 3000, false);
	end
	if MovableMan:IsActor(self.PlayerActor) then
		if self.DoorBox:WithinBox(self.PlayerActor.Pos) and (self.KilledSoldiers ~= true) then
			FrameMan:ClearScreenText(0);
			self:ResetMessageTimer(0);
			FrameMan:SetScreenText("Hm...  Whoever's inside won't let me in.  Maybe if I kill all the Coalition he'll be polite?", 0, 0, 3000, false);
		end
	end
	if MovableMan:IsActor(self.PlayerActor) then
		if self.Flashing == true then
			if self.FlashTimer:IsPastSimMS(400) then
				self.Flashing = false;
			else
				self.PlayerActor:FlashWhite(1);
			end
		end
	end
	
	if MovableMan:IsActor(self.PlayerActor) then
		if self.PlayerActor.Pos.X > 3000 and self.KilledSoldiers == true then
			self:AdvanceToTwo();
		end
	end
end

-----------------------------------------------------------------------------------------
-- Advance To Two
-----------------------------------------------------------------------------------------
function SweetDreams:AdvanceToTwo()
	-- Switch to next phase
	self.CurrentFightPhase = self.FightPhase.PHASETWO;
	AudioMan:PlayMusic("SweetDreams.rte/Sounds/Castle.ogg", -1, -1);
	self.PlayerActor.Pos = Vector(3000, 650);
	self.PlayerActor.Vel = Vector(0, 0);
	self.brain.Pos = Vector(3500, 480);
	self.ForcedControlledActor = self.brain;
	self.PlayerActor:SetControllerMode(Controller.CIM_NETWORK,-1);
end


-----------------------------------------------------------------------------------------
-- Phase Two (Outro)
-----------------------------------------------------------------------------------------
-- This phase is where I put the outro.
function SweetDreams:PhaseTwo()

	if UInputMan:KeyPressed(26) then
		self:SkipOutro();
	end
	
	-- See the comment above where self.ViewAngle is defined for why I must do this.
	self.PlayerActor:SetAimAngle(self.ViewAngle);

	if self.OutroStage == -2 then -- (I didn't want to have to rearrange the numbers)
		self:ResetMessageTimer(0);
		--FrameMan:ClearScreenText(0);
		FrameMan:SetScreenText("(Press 'z' to skip)", 0, 0, 1000, false);
		self.OutroStage = -1;
	end
	if self.OutroTimer:IsPastSimMS(2000) and self.OutroStage == -1 then
		self:ResetMessageTimer(0);
		self.OutroStage = 0;
	end

	-- Walk towards the castle, looking up slightly
	if self.OutroStage <= 1 then
		if self.PlayerActor.Pos.X <= 3460 then
			self.PlayerActor:GetController():SetState(Controller.MOVE_RIGHT, true);
			--Talk when almost there.
			if self.PlayerActor.Pos.X >= 3200 and self.OutroStage == 0 then
				self:ResetMessageTimer(0);
				FrameMan:SetScreenText("Finally, the castle.", 0, 0, 3000, false);
				self.ViewAngle = 0.5;
				self.OutroStage = 1;
			end
		--When you get there, stop and look forward.
		else
			self.OutroStage = 2;
			self.ViewAngle = (3.14/2)-0.6;
			-- Don't forget to reset the timer, because after all it can take varying times to get there.
			self.OutroTimer:Reset();
		end
	end
	
	-- Pause for a second, then talk.  He looks around in various ways too.
	if self.OutroTimer:IsPastSimMS(1000) and self.OutroStage == 2 then
		self.OutroStage = 3;
		self:ResetMessageTimer(0);
		FrameMan:SetScreenText("Wait a second...  I knew this looked familiar...", 0, 0, 3000, false);
	end
	if self.OutroTimer:IsPastSimMS(6000) and self.OutroStage == 3 then
		self.OutroStage = 4;
		self.ViewAngle = 0;
	end
	if self.OutroTimer:IsPastSimMS(8000) and self.OutroStage == 4 then
		self.OutroStage = 5;
		self:ResetMessageTimer(0);
		FrameMan:SetScreenText("No.", 0, 0, 3000, false);--"Shit.", 0, 0, 3000, false);
	end
	if self.OutroTimer:IsPastSimMS(12000) and self.OutroStage == 5 then
		self.OutroStage = 6;
		self:ResetMessageTimer(0);
		FrameMan:SetScreenText("This is Super Mario World!", 0, 0, 3000, false); --, damnit!", 0, 0, 3000, false);
	end
	if self.OutroTimer:IsPastSimMS(13500) and self.OutroStage == 6 then
		self.OutroStage = 7;
	end
	-- Walk to the door.
	if self.OutroStage == 7 then
		if self.PlayerActor.Pos.X <= 3700-(128/2) then
			self.PlayerActor:GetController():SetState(Controller.MOVE_RIGHT, true);
		else
			-- Don't forget to reset the timer, because after all it can take varying times to get there.
			self.OutroTimer:Reset();
			self.OutroStage = 8;
			self:ResetMessageTimer(0);
			FrameMan:SetScreenText("I couldn't even beat it on the damn console...", 0, 0, 3000, false);
		end
	end
	if self.OutroTimer:IsPastSimMS(5000) and self.OutroStage == 8 then
		self.OutroStage = 9;
		self:ResetMessageTimer(0);
		FrameMan:SetScreenText("This is going to be tough...  Great, now the door won't even open!", 0, 0, 3000, false);
	end
	if self.OutroTimer:IsPastSimMS(11000) and self.OutroStage == 9 then
		self.OutroStage = 10;
		self:ResetMessageTimer(0);
		FrameMan:SetScreenText("So what'd Mario do when he entered castles in Super Mario World?", 0, 0, 3000, false);
	end
	if self.OutroTimer:IsPastSimMS(15500) and self.OutroStage == 10 then
		self.OutroStage = 11;
		self:ResetMessageTimer(0);
		FrameMan:SetScreenText("...", 0, 0, 3000, false);
	end
	if self.OutroTimer:IsPastSimMS(21000) and self.OutroStage == 11 then
		self.OutroStage = 12;
		self:ResetMessageTimer(0);
		FrameMan:SetScreenText("Oh yeah.  He looked up for some reason.  It's worth a try...", 0, 0, 3000, false);
	end
	if self.OutroTimer:IsPastSimMS(23500) and self.OutroStage == 12 then
		self.OutroStage = 13;
		self.ViewAngle = 3.14/2;
	end
	if self.OutroTimer:IsPastSimMS(25000) and self.OutroStage == 13 then
		self.OutroStage = 14;
		self.ViewAngle = 0;
	end
	if self.OutroTimer:IsPastSimMS(26500) and self.OutroStage == 14 then
		self.OutroStage = 15;
		self.DoorTimer:Reset();
		self.DoorSound = CreateAEmitter("SweetDreams.rte/Door Sound");
		self.DoorSound.Pos = self.Door.Pos;
		MovableMan:AddParticle(self.DoorSound);
	end
	if self.DoorTimer:IsPastSimMS(160) and self.OutroStage == 15 then
		if self.Door.Frame < 7 then
			self.Door.Frame = self.Door.Frame + 1;
			self.DoorTimer:Reset();
		else
			self.DoorTimer:Reset();
			self.OutroTimer:Reset();
			self.OutroStage = 16;
		end
	end
	if self.OutroTimer:IsPastSimMS(2000) and self.OutroStage == 16 then
		self.OutroStage = 17;
		self:ResetMessageTimer(0);
		FrameMan:SetScreenText("Neat!  ...I guess it's a secret code or something?", 0, 0, 3000, false);
	end
	if self.OutroStage == 17 then
		if self.PlayerActor.Pos.X <= 3700+(128/2) - 24 then
			self.PlayerActor:GetController():SetState(Controller.MOVE_RIGHT, true);
		else
			-- Don't forget to reset the timer, because after all it can take varying times to get there.
			self.OutroTimer:Reset();
			self.OutroStage = 18;
			self.PlayerActor.Pos = Vector(45,960); --Hide Dimitri in the LOL cave.
			--self:ResetMessageTimer(0);
			--FrameMan:SetScreenText("I wasn't even able to beat it on the stupid console...", 0, 0, 3000, false);
		end
	end
	if self.OutroTimer:IsPastSimMS(3000) and self.OutroStage == 18 then
		self.OutroStage = 19;
		self.DoorTimer:Reset();
	end
	if self.DoorTimer:IsPastSimMS(160) and self.OutroStage == 19 then
		if self.Door.Frame > 0 then
			self.Door.Frame = self.Door.Frame - 1;
			self.DoorTimer:Reset();
		else
			--self.DoorTimer:Reset();
			self.OutroTimer:Reset();
			self.OutroStage = 20;
			
		end
	end
	if self.OutroTimer:IsPastSimMS(2000) and self.OutroStage == 20 and self.ActivityState ~= Activity.OVER then
		self:ResetMessageTimer(0);
		FrameMan:SetScreenText("Will Dimitri find his cake?  Find out in episode 2, coming soon-ish!  And 3, coming later!", 0, 0, 5000, false);
		self.WinnerTeam = Activity.TEAM_1;
		ActivityMan:EndActivity();
	end

end

function SweetDreams:PhaseThree()
end



-----------------------------------------------------------------------------------------
-- Gameover conditions check
-----------------------------------------------------------------------------------------
-- Here we check if we stil have a brain to control our troops. If not - game over
function SweetDreams:Check()
	-- Check if mission is still running to prevent mission failure
	-- if you waste brain after victory or lose it's object after evacuation
	if self.ActivityState ~= Activity.OVER then
		if MovableMan:IsActor(self.PlayerActor) then
			--If the player is alive, set its last position to this number.
			self.lastpos = self.PlayerActor.Pos;
			--Set the observation target, or what the game flies to when the game ends, to the player's last position.
            --self:SetObservationTarget(self.lastpos, Activity.PLAYER_1);
		end
	end
	if self.ActivityState ~= Activity.OVER and not MovableMan:IsActor(self.PlayerActor) then
		self.Pies.ToDelete = true;
		-- Steal one of Dimitri's lives
		self:SetTeamFunds(self:GetTeamFunds(self.PlayerTeam) - 1, self.PlayerTeam);
		if self:GetTeamFunds(self.PlayerTeam) <= 0 then
			-- Game over.
			self.WinnerTeam = Activity.TEAM_2;
			self:ResetMessageTimer(0);
			FrameMan:SetScreenText("Dimitri ran out of gold...", Activity.TEAM_1, 0, -1, false);
			self:SetObservationTarget(self.lastpos, Activity.PLAYER_1);
			ActivityMan:EndActivity();
		else
			for i = 0, self.CheckpointAmount - 1 do -- (-1 because tables include zero)
				local Checkpoint = self.Checkpoints[i];
				if Checkpoint.Mass == 1 then
					self.SavedCP = Checkpoint;
					-- We've found the saved one, what's the point of looping through the others?
					break;
				end
			end
			self.PlayerActor = CreateAHuman("Dimitri","Ronin.rte");
			self.PlayerActor.AIMode = Actor.AIMODE_SENTRY;
			self.PlayerActor:SetControllerMode(Controller.CIM_NETWORK,-1);
			self.PlayerActor.Team = Activity.TEAM_1;
			self.Pies = CreateHDFirearm("Pies","SweetDreams.rte");
			self.PlayerActor:AddInventoryItem(self.Pies);
			if self.SavedCP ~= nil then
				self.PlayerActor.Pos = self.SavedCP.Pos + Vector(4, 0);
			else --If no CPs are activated, send him to his sleeping bag
				self.PlayerActor.Pos = Vector(110,550);
			end
			controller = self.PlayerActor:GetController();
			self.ForcedControlledActor = self.PlayerActor;
			self.FlashTimer:Reset();
			self.Flashing = true;
			MovableMan:AddActor(self.PlayerActor);
		end
	end


end

function SweetDreams:GetActorToSave()
	return self.PlayerActor;
end
function SweetDreams:ResetCPs()
	for i = 0, self.CheckpointAmount - 1 do -- (-1 because tables include zero)
		local Checkpoint = self.Checkpoints[i];
		--Checkpoint:UnSave(Checkpoint);
		-- Can't find any other way of doing it...  I set
		-- mass to zero to tell the checkpoint to turn saved
		-- off.  It's pinned, so it won't behave different.
		Checkpoint.Mass = 0;
		Checkpoint.Frame = 0;
	end
end
-----------------------------------------------------------------------------------------
-- Mission objective "Sweep" update state
-----------------------------------------------------------------------------------------
function SweetDreams:ObjectiveSweep()
	if not self.KilledSoldiers then
		
		local liveEnemies = 0;
		
		-- Why do we need two loops here you ask? That't quite tricky. The problem is that when you add actors in Init
		-- phase they are placed in MovableMan.AddedActors collection first and then after a short while they are 
		-- moved to MovableMan.Actors. If we enumerate only MovableMan.Actors we'll get winning condition triggered on
		-- start, because this collection will be empty. So we need to enumerate both of them. I can think of other 
		-- ways to avoid this. Check for winning conditions after a short while after start (250 ms is enough). Initialize
		-- objectives inside UpdateActivity function or maybe some others. But they are hacky enough to choose both 
		-- collection enumeration.
		for actor in MovableMan.AddedActors do
			-- Exclude doors. They are actors too.
			if actor.ClassName ~= "ADoor" and actor.Team == self.CPUTeam then
				liveEnemies = liveEnemies + 1;
				--self:AddObjectivePoint("Kill All", actor.AboveHUDPos, self.PlayerTeam, GameActivity.ARROWDOWN);
				actor.AIMode = Actor.AIMODE_SENTRY;
				actor:GetController():SetState(Controller.BODY_CROUCH,false);
				actor:GetController():SetState(Controller.MOVE_LEFT,false);
				actor:GetController():SetState(Controller.MOVE_RIGHT,false);
			end
		end -- for
		
		for actor in MovableMan.Actors do
			if actor.ClassName ~= "ADoor" and actor.Team == self.CPUTeam then
				liveEnemies = liveEnemies + 1;
				--self:AddObjectivePoint("Kill All", actor.AboveHUDPos, self.PlayerTeam, GameActivity.ARROWDOWN);
				actor.AIMode = Actor.AIMODE_SENTRY;
				actor:GetController():SetState(Controller.BODY_CROUCH,false);
				actor:GetController():SetState(Controller.MOVE_LEFT,false);
				actor:GetController():SetState(Controller.MOVE_RIGHT,false);
			end -- if
		end --for
		
		if true or liveEnemies == 0 then
			print("Sweep complete!")
			self.KilledSoldiers = true;
		end -- if
	end -- if
end

function SweetDreams:AddCoalition(x, y, flipped, heavy, weaponName)
	if heavy == true then
		self.Soldier = CreateAHuman("Soldier Heavy","Coaliton.rte");
	else
		self.Soldier = CreateAHuman("Soldier Light","Coaliton.rte");
	end
	self.Soldier.AIMode = Actor.AIMODE_SENTRY;
	self.Soldier.Team = self.CPUTeam;
	weapon = CreateHDFirearm(weaponName,"Coalition.rte");
	self.Soldier:AddInventoryItem(weapon);
	self.Soldier.Pos = Vector(x,y);
	self.Soldier.HFlipped = flipped;
	MovableMan:AddActor(self.Soldier);
end

function SweetDreams:SkipIntro()
	FrameMan:ClearScreenText(0);
	self.PlayerActor.Pos = Vector(170,550); -- Bring Dimitri back from the LOL cave
	self.PlayerActor.Vel = Vector(0,0);
	if self.IntroStage < 7 then
		self.ScrapRight = CreateMOSRotating("Paper Right");
		self.ScrapRight.Pos = Vector(270,568);
		self.ScrapRight.RotAngle = 0.2;
		MovableMan:AddMO(self.ScrapRight);
		self.ScrapLeft = CreateMOSRotating("Paper Left");
		self.ScrapLeft.Pos = Vector(200,568);
		self.ScrapLeft.RotAngle = -0.3;
		MovableMan:AddMO(self.ScrapLeft);
		self.Note.ToDelete = true;
		
		--lava test
		for i = 0, 8 do
			self.Lava = CreateMOSRotating("Lava");
			self.Lava.Pos = Vector(350 + 16*i,590);
			MovableMan:AddMO(self.Lava);
		end
		if self.IntroStage > 3 then
			self.WeaponSwitch = true;
		end
	end
	self.DimitriBag.Frame = 3;
	self.PlayerActor:SetAimAngle(0);
	self:AdvanceToOne();
end

function SweetDreams:SkipOutro()
	if self.SkippedOutroYet == false then
		self.SkippedOutroYet = true;
		self.PlayerActor.Pos = Vector(45,960); --Hide Dimitri in the LOL cave.
		self.Door.Frame = 0;
		self.OutroStage = 20;
		self.OutroTimer:Reset();
		self:ResetMessageTimer(0);
		FrameMan:SetScreenText("Will Dimitri find his cake?  Find out in episode 2, coming soon-ish!  And 3, coming later!", 0, 0, 5000, false);
		self.WinnerTeam = Activity.TEAM_1;
		ActivityMan:EndActivity();
	end
end